package org.sonatype.nexus.plugin.gwt.client;

import java.util.List;

import org.sonatype.nexus.plugin.gwt.client.model.FormError;

import com.extjs.gxt.ui.client.binding.FieldBinding;
import com.extjs.gxt.ui.client.binding.FormBinding;
import com.extjs.gxt.ui.client.data.BaseLoader;
import com.extjs.gxt.ui.client.data.LoadEvent;
import com.extjs.gxt.ui.client.data.Loader;
import com.extjs.gxt.ui.client.data.ModelData;
import com.extjs.gxt.ui.client.data.ModelType;
import com.extjs.gxt.ui.client.event.Listener;
import com.extjs.gxt.ui.client.widget.LayoutContainer;
import com.extjs.gxt.ui.client.widget.TabItem;
import com.extjs.gxt.ui.client.widget.TabPanel;
import com.extjs.gxt.ui.client.widget.treepanel.TreePanel;
import com.extjs.gxt.ui.client.widget.form.FormPanel;
import com.extjs.gxt.ui.client.widget.layout.FormLayout;
import com.extjs.gxt.ui.client.widget.treepanel.TreePanel;
import com.google.gwt.core.client.GWT;
import com.google.gwt.http.client.RequestBuilder;
import com.google.gwt.http.client.RequestBuilder.Method;
import com.google.gwt.http.client.Response;
import com.google.gwt.user.client.Window;

public abstract class AbstractPluginPanel <M extends ModelData> extends LayoutContainer
{
    private FormBinding formBindings;
    private FormPanel formPanel;
    //private TreePanel treePanel;

    public AbstractPluginPanel()
    {   
        // just the UI bits
        setLayout( new FormLayout() );
       

        formPanel = this.createFormPanel();  
       
    	//treePanel = this.createTreePanel();
    	  
    	
        // binding
        this.formBindings = new FormBinding( formPanel, true );
        this.formBindings.autoBind();

        // populate, we should do this onLoad ?
        sendMessageToRemote( RequestBuilder.GET );
    }
    
    protected abstract String getServiceUrl();
    
    protected abstract FormPanel createFormPanel();
    
   // protected abstract TreePanel createTreePanel();
    
    protected abstract ModelType getModelType();
    
    protected abstract ModelData createNewModelInstance();
    
    protected void sendMessageToRemote( Method method )
    {
        // setup the URL and headers
        RequestBuilder builder = new RequestBuilder( method, GWT.getHostPageBaseURL() + getServiceUrl() );
        builder.setHeader( "Accept", "application/json" );
        builder.setHeader( "Content-Type", "application/json" );
        builder.setHeader( "Cache-Control", "no-cache" );
        builder.setHeader( "Expires", "no-cache" );

        // only send payload for POST and PUT
        if ( (RequestBuilder.POST.equals( method ) || RequestBuilder.PUT.equals( method ) ) )
        {
            if( formPanel.isValid() )
            {
                JsonWriter jsonWriter = new JsonWriter( getModelType() );
                String jsonString = jsonWriter.toJson( formBindings.getModel() );
                builder.setRequestData( jsonString );
            }
            else
            {
                // the form is not valid
                return;
            }
        }

        // custom proxy so we do NOT loose to request data
        RequestBuilderDataProxy<M> proxy = new RequestBuilderDataProxy<M>( builder );

        // JSON Reader needs to know which type it is creating
        JsonReader<M> jsonReader = new JsonReader<M>( this.getModelType() )
        {
            @Override
            protected ModelData newModelInstance()
            {
                return createNewModelInstance();
            }
        };

        // loader binds to the form
        final BaseLoader<M> loader = new BaseLoader<M>( proxy, jsonReader );
        loader.addListener( Loader.Load, new Listener<LoadEvent>()
        {
            public void handleEvent( LoadEvent event )
            {
                formBindings.bind( (ModelData) event.getData() );
            }
        } );

        // handle failure
        loader.addListener( Loader.LoadException, new Listener<LoadEvent>()
        {
            public void handleEvent( LoadEvent event )
            {

                boolean handled = false;
                
                if( event.exception instanceof RemoteFailureException )
                {
                    RemoteFailureException exception = (RemoteFailureException) event.exception;
                    Response response = exception.getResponse();
                    
                    // TODO: is there a better way to do this?
                    if( response.getText().contains( "{\"errors\":" ) )
                    {
                        JsonReader<List<FormError>> errorReader = new JsonReader<List<FormError>>( FormError.getModelType() )
                        {
                            @Override
                            protected ModelData newModelInstance()
                            {
                                return new FormError();
                            }
                        };
                        
                        // clear current errors
                        for( FieldBinding fieldBinding : formBindings.getBindings() )
                        {
                            fieldBinding.getField().clearInvalid();
                        }
                        
                        List<FormError> errors = errorReader.read( null, response.getText() );
                        
//                        JsonWriter jsonWriter = new JsonWriter( FormError.getModelType() );
                        for ( FormError formError : errors )
                        {
//                            System.out.println( "error: "+ jsonWriter.toJson( formError ) );
                            
                            boolean found = false;
                            
                            for( FieldBinding fieldBinding : formBindings.getBindings() )
                            {
                                if( fieldBinding.getProperty().equals( formError.getId() ))
                                {
                                    fieldBinding.getField().forceInvalid( formError.getMessage() );
                                    // found it, continue with the next error
                                   found = true;
                                   break;   
                                }
                            }
                            
                            if( !found )
                            {
                                // FIXME
                                // set error message 
                            }
                        }
                        
                        handled = true;
                    }
                }
                
                if( !handled )
                {
                    Window.alert( "Failed connect to server: " + event.exception.getMessage() );
                }
            }
        } );

        // call do load makes the remote request
        loader.load();
    }
    
}
